@@ -17,7 +17,7 @@
 
 #include "keducaview.h"
 #include "keducaview.moc"
-
+#include "settings.h"
 #include "../libkeduca/fileread.h"
 #include "kquestion.h"
 #include "kgroupeduca.h"
@@ -28,14 +28,19 @@
 #include <kstandarddirs.h>
 #include <kmessagebox.h>
 #include <kfiledialog.h>
+#include <kpushbutton.h>
 #include <kdebug.h>
 #include <kio/netaccess.h>
 
 #include <qtimer.h>
+#include <qlayout.h>
+#include <qlabel.h>
+#include <qtextedit.h>
 
-KEducaView::KEducaView(QWidget *parent, const char *name ) : QWidget(parent,name),
-                                                             _timeoutTimer(0L)
+KEducaView::KEducaView( QWidget *parent, const char *name )
+    : QWidgetStack( parent, name ), _keducaFile( 0 ), _timeoutTimer( 0 )
 {
+    init();
 }
 
 KEducaView::~KEducaView()
@@ -52,41 +57,57 @@ KEducaView::~KEducaView()
 /** Init graphical interface */
 void KEducaView::init()
 {
-    QString infoStart = "";
-    QVBoxLayout *vbox = new QVBoxLayout( this );
-    vbox->setSpacing( 6 );
-    vbox->setMargin( 11 );
+    // Intro Screen
+    _introWidget = new QLabel( this, "introScreen" );
+    _introWidget->setBackgroundColor( Qt::white );
+    _introWidget->setAlignment( AlignCenter );
+    _introWidget->setPixmap( QPixmap( locate("data","keduca/pics/keduca_init.png") ) );
+    addWidget( _introWidget, 0 );
 
-    _vbox2 = new QVBoxLayout( vbox );
-    _split = new QSplitter( this );
-    _split->setOrientation( QSplitter::Vertical );
+    // Info Widget
+    _infoWidget = new QVBox( this, "infoWidget" );
+    _viewInfo = new QTextEdit( _infoWidget );
+    _viewInfo->setReadOnly( true );
+    _buttonStartTest = new KPushButton( i18n( "&Start Test" ), _infoWidget );
+    connect( _buttonStartTest, SIGNAL( clicked() ), SLOT( slotButtonNext() ) );
+    addWidget( _infoWidget, 1 );
+
+    // Question Widget
+    _questionWidget = new QVBox( this, "questionWidget" );
+    _split  = new QSplitter( QSplitter::Vertical, _questionWidget );
     _questionText = new KQuestion( _split, "kquestion" );
     _buttonGroup = new KGroupEduca( _split, "ButtonGroup" );
-    _buttonGroup->setRadioButtonExclusive( TRUE );
-    _vbox2->addWidget( _split );
-    _questionText->hide();
-    _split->hide();
-
-    _viewResults = new QTextView(this);
-    vbox->addWidget( _viewResults );
-
-    _buttonSave = new QPushButton( this, "ButtonSave" );
-    _buttonSave->setText( i18n("&Save Results...") );
-    _buttonSave->setAutoRepeat( FALSE );
-    _buttonSave->setToggleButton( FALSE );
-    _buttonSave->setDefault( TRUE );
-    _buttonSave->hide();
-    connect( _buttonSave, SIGNAL( clicked() ), SLOT( slotButtonSave() ) );   
-    vbox->addWidget( _buttonSave );
-    
-    _buttonNext = new QPushButton( this, "ButtonNext" );
-    _buttonNext->setText( i18n("&Next >>") );
-    _buttonNext->setAutoRepeat( FALSE );
-    _buttonNext->setToggleButton( FALSE );
-    _buttonNext->setDefault( TRUE );
+    _buttonGroup->setRadioButtonExclusive( true );
+    _buttonNext = new KPushButton( i18n( "&Next >>" ), _questionWidget, "ButtonNext" );
     connect( _buttonNext, SIGNAL( clicked() ), SLOT( slotButtonNext() ) );
-    vbox->addWidget( _buttonNext );
-    _viewResults->setText( getInformation() );
+    addWidget( _questionWidget, 2 );
+
+    // Results Widget
+    _resultsWidget = new QVBox( this, "resultsWidget" );
+    _viewResults = new QTextEdit( _resultsWidget );
+    _viewResults->setReadOnly( true );
+    _buttonSave = new KPushButton( i18n( "&Save Results..." ), _resultsWidget );
+    connect( _buttonSave, SIGNAL( clicked() ), SLOT( slotButtonSave() ) );
+    /* FIXME: This is a hack
+     * The results widget can be shown only in the end of the test (when you should be shown the
+     * option to save the results) or at the middle of the test, after each question (when there should
+     * be a "Next" button).
+     */
+    _buttonResultsNext = new KPushButton( i18n( "&Next >>" ), _resultsWidget );
+    connect( _buttonResultsNext, SIGNAL( clicked() ), SLOT( slotButtonNext() ) );
+    _buttonResultsNext->hide();
+    addWidget( _resultsWidget, 3 );
+
+    raiseWidget( _introWidget );
+
+    // Restore splitter size
+    _split->setSizes( Settings::splitter_size() );
+}
+
+void KEducaView::slotButtonStartTest()
+{
+    raiseWidget( _questionWidget );
+    showRecord();
 }
 
 /** Button Next action */
@@ -100,13 +121,16 @@ void KEducaView::slotButtonNext()
       _questionText->countdown(0);
     }
 
-    if( _questionText->isVisible() && !_isInitStatus )
+    if( ( visibleWidget() == _questionWidget ) && !_isInitStatus )
       setResults();
 
     _buttonGroup->clearAnswers();
 
-    if( !_showResultFinish && _questionText->isVisible() )
+    if( ( Settings::resultFinish()== Settings::EnumResultFinish::afterEachQuestion ) 
+        && _questionText->isVisible() )
+    {
         showResults( _currentResults + "<HR>" + currentStatusPoints() );
+    }
     else
     {
         if( questionNext() )
@@ -114,9 +138,8 @@ void KEducaView::slotButtonNext()
         else
           {
           configWrite();
-          _buttonNext->hide();
-          _buttonSave->show();
-          showResults( setFinalResult() + currentStatusPoints() + "<HR><P>" + _results + "</HTML>" );
+          showResults( setFinalResult() + currentStatusPoints() + "<HR><P>" + _results + "</HTML>"
+);
           }
     }
 }
@@ -150,7 +173,6 @@ void KEducaView::slotButtonSave()
         if ( QFileInfo( newURL.path() ).extension().isEmpty() ) {
             // No more extensions in filters. We need to get it from the mimetype.
             QString extension = ".html";
-            kdDebug() << "KControlDoc::slotFileSaveAs outputFormat=" << outputFormat << " extension=" << extension << endl;
             newURL.setPath( newURL.path() + extension );
         }
 
@@ -179,16 +201,7 @@ void KEducaView::showRecord()
 {
     QString questionTextTemp = "";
     
-    if( _viewResults->isVisible() )
-    {
-        _viewResults->hide();
-        _buttonSave->hide();
-        _questionText->show();
-        _split->show();
-    }
-   
-    if( _keducaFile->getQuestionInt(FileRead::QF_TIME) >0 )
-      KMessageBox::information(this, i18n("You have %1 seconds to complete this question.\n\nPress OK when you are ready.").arg( _keducaFile->getQuestion(FileRead::QF_TIME) ) );
+    raiseWidget( _questionWidget );
     
     _keducaFileIndex++;
     bool MultiAnswer = _keducaFile->isMultiAnswer();
@@ -217,7 +230,7 @@ void KEducaView::showRecord()
         _buttonGroup->setType( KGroupEduca::Radio );
 
 
-    if( _showRandomAnswers )
+    if( Settings::randomAnswers() )
       {
       _randomAnswers.clear();
       QValueList<unsigned int> tmpRandom;
@@ -245,8 +258,17 @@ void KEducaView::showRecord()
             _keducaFile->recordAnswerNext();
         };
       }
+
     
     // START THE TIMER *****************************************************************
+    if( _keducaFile->getQuestionInt(FileRead::QF_TIME) > 0 )
+    {
+        KMessageBox::information( this,
+                                  i18n( "You have %1 seconds to complete this question.\n\n"
+                                        "Press OK when you are ready." )
+                                  .arg( _keducaFile->getQuestion(FileRead::QF_TIME) ) );
+     }
+
     int timeout = _keducaFile->getQuestionInt(FileRead::QF_TIME);
     if (timeout > 0)
     {
@@ -259,7 +281,9 @@ void KEducaView::showRecord()
       _timeoutTimer->start(1000*timeout);
       _questionText->countdown(timeout);
       _questionText->countdownVisible(true);
-    }else{
+    }
+    else
+    {
       _questionText->countdownVisible(false);
     }
 }
@@ -273,9 +297,20 @@ void KEducaView::questionTimedOut()
 /** Show results */
 void KEducaView::showResults( const QString &text )
 {
-    _split->hide();
     _viewResults->setText( text );
-    _viewResults->show();
+   
+    if ( _keducaFile->recordEOF() )
+    {
+        _buttonSave->show();
+	   _buttonResultsNext->hide();
+    }
+    else
+    {
+        _buttonSave->hide();
+	   _buttonResultsNext->show();
+    }
+    
+    raiseWidget( _resultsWidget );
 }
 
 /** Set results */
@@ -286,7 +321,7 @@ void KEducaView::setResults()
     QString correctAnswer = "";
     QValueList<uint>::iterator itRandom = _randomAnswers.begin();
     
-    (_showRandomAnswers) ? _keducaFile->recordAnswerAt(*itRandom) : _keducaFile->recordAnswerFirst();
+    Settings::randomAnswers() ? _keducaFile->recordAnswerAt(*itRandom) : _keducaFile->recordAnswerFirst();
 
     for( unsigned index = 0 ; index < _keducaFile->recordAnswerCount() ; index++ )
     {
@@ -305,7 +340,7 @@ void KEducaView::setResults()
                 if(_keducaFile->getAnswerValue()) correctAnswer += "<BR><FONT COLOR=#006b6b><B>" + answertext + "</B></FONT>";
             }
         }
-        if(_showRandomAnswers)
+        if( Settings::randomAnswers() )
           {
           ++itRandom;
           _keducaFile->recordAnswerAt(*itRandom);
@@ -366,7 +401,8 @@ QString KEducaView::setFinalResult()
 bool KEducaView::openURL( const KURL &url)
 {
     _keducaFile = new FileRead();
-    if( !_keducaFile->openFile( url ) ) return false;
+    if ( !_keducaFile->openFile( url ) )
+        return false;
 
     _isInitStatus = true;
     _results = "<HTML>";
@@ -377,10 +413,7 @@ bool KEducaView::openURL( const KURL &url)
     _incorrectPoints = 0;
     _keducaFileIndex = 0;
 
-    init();
-    configRead();
-
-    if( _showRandomQuestions )
+    if( Settings::randomQuestions() )
       {
       unsigned int index;
       for( index = 0 ; index < _keducaFile->getTotalQuestions() ; index ++ )
@@ -389,28 +422,18 @@ bool KEducaView::openURL( const KURL &url)
 
     _keducaFile->recordFirst();
     _keducaFile->recordAnswerFirst();
+    
+    _viewInfo->setText( getInformation() );
+    raiseWidget( _infoWidget );
+    
     return true;
 }
 
-/** Read settings */
-void KEducaView::configRead()
-{
-    KConfig *config = KGlobal::config();
-    config->setGroup( "keducaview" );
-    _split->setSizes( config->readIntListEntry("Splitter_size") );
-    config->setGroup("Options");
-    _showResultFinish = config->readBoolEntry("ResultFinish", true );
-    _showRandomQuestions = config->readBoolEntry("RandomQuestions", false );
-    _showRandomAnswers = config->readBoolEntry("RandomAnswers", false );    
-}
-
 /** Write settings */
 void KEducaView::configWrite()
 {
-    KConfig *config = KGlobal::config();
-    config->setGroup( "keducaview" );
-    config->writeEntry("Splitter_size", _split->sizes() );
-    config->sync();
+    Settings::setSplitter_size( _split->sizes() );
+    Settings::writeConfig();
 }
 
 /** current Status Points */
@@ -447,9 +470,9 @@ QString KEducaView::getTableQuestion( bool isCorrect, const QString &correct, co
 {
   _currentResults = "<TABLE WIDTH=100% BORDER=0><TR><TD VALIGN=TOP WIDTH=70><IMG WIDTH=64 HEIGHT=64 SRC=";                     
   if( isCorrect )
-    _currentResults += locate("appdata","pics/keduca_correct_64.png");
+    _currentResults += locate("data","keduca/pics/keduca_correct_64.png");
     else
-    _currentResults += locate("appdata","pics/keduca_incorrect_64.png");
+    _currentResults += locate("data","keduca/pics/keduca_incorrect_64.png");
 
   _currentResults += "></TD><TD VALIGN=TOP>";
 
@@ -480,7 +503,7 @@ QString KEducaView::getTableQuestion( bool isCorrect, const QString &correct, co
 /** Define the next questions (random, secuencial) */
 bool KEducaView::questionNext()
 {
-  if( !_showRandomQuestions && !_isInitStatus )
+  if( !Settings::randomQuestions() && !_isInitStatus )
     {
     _keducaFile->recordNext();
     return _keducaFile->recordEOF() ? false : true;
@@ -489,7 +512,7 @@ bool KEducaView::questionNext()
   if( _isInitStatus )
     {
     _isInitStatus = false;
-    if( !_showRandomQuestions ) return true;
+    if( !Settings::randomQuestions() ) return true;
     }
  
   if( _randomQuestions.count() > 0 )
@@ -509,22 +532,33 @@ bool KEducaView::questionNext()
 /** Get document information */
 QString KEducaView::getInformation()
 {
-  QString tmp = "";
 
-  if( !(_keducaFile->getHeader("startupinfo")).isEmpty() ){
-    tmp += insertTable( "<img src=" + locate("appdata","pics/alert.png") + ">", 2 )
-//        + insertRow( i18n("Additional Information")+"<br>", true, 2 )
-        + insertRow( _keducaFile->getHeader("startupinfo") + "<br>", false )
-        + insertTableClose();
+  QString tmp = "";
+  
+  if ( _keducaFile )
+  {
+    if( !(_keducaFile->getHeader("startupinfo")).isEmpty() )
+    {
+      tmp += insertTable( "<img src=" + locate("data","keduca/pics/alert.png") + ">", 2 )
+//          + insertRow( i18n("Additional Information")+"<br>", true, 2 )
+          + insertRow( _keducaFile->getHeader("startupinfo") + "<br>", false )
+          + insertTableClose();
     }
 
-  tmp += insertTable( "<img src=" + locate("appdata","pics/info.png") + ">", 2 );
-  tmp += insertRow( i18n("Information")+"<br>", true, 2);
-  tmp += insertRow( i18n("Title"), _keducaFile->getHeader("title"), true );
-  tmp += insertRow( i18n("Category"), _keducaFile->getHeader("category"), true );
-  tmp += insertRow( i18n("Type"), _keducaFile->getHeader("type"), true );
-  tmp += insertRow( i18n("Language"), _keducaFile->getHeader("language"), true );
-  tmp += insertTableClose();
+    tmp += insertTable( "<img src=" + locate("data","keduca/pics/info.png") + ">", 2 );
+    tmp += insertRow( i18n("Information")+"<br>", true, 2);
+    tmp += insertRow( i18n("Title"), _keducaFile->getHeader("title"), true );
+    tmp += insertRow( i18n("Category"), _keducaFile->getHeader("category"), true );
+    tmp += insertRow( i18n("Type"), _keducaFile->getHeader("type"), true );
+    tmp += insertRow( i18n("Language"), _keducaFile->getHeader("language"), true );
+    tmp += insertTableClose();
+  }
+  else
+  {
+    tmp += "<html><body>";
+    tmp += "<img src=" + locate("data","keduca/pics/keduca_init.png") + "/>";
+    tmp += "</body></html>";
+  }
 
   return tmp;
 }
