@@ -1,4 +1,3 @@
-// native-filter.cc
 // Copyright (C)  2003  Dominique Devriese <devriese@kde.org>
 
 // This program is free software; you can redistribute it and/or
@@ -33,10 +32,17 @@
 
 #include "config.h"
 
+#include <qdom.h>
 #include <qfile.h>
 #include <qregexp.h>
-#include <qdom.h>
+
+#include <karchive.h>
+#include <kdebug.h>
 #include <kglobal.h>
+#include <kstandarddirs.h>
+#include <ktar.h>
+
+#include <stdio.h>
 
 #include <vector>
 #include <algorithm>
@@ -104,10 +110,64 @@ KigDocument* KigFilterNative::load( const QString& file )
     fileNotFound( file );
     return 0;
   };
-  QDomDocument doc( "KigDocument" );
-  if ( !doc.setContent( &ffile ) )
+
+  QFile kigdoc( file );
+#ifndef KIG_NO_COMPRESSED_FILES
+  bool iscompressed = false;
+  if ( !file.endsWith( ".kig", false ) )
+  {
+    // the file is compressed, so we have to decompress it and fetch the
+    // kig file inside it...
+    iscompressed = true;
+
+    QString tempdir = KGlobal::dirs()->saveLocation( "tmp" );
+    if ( tempdir.isEmpty() )
+      KIG_FILTER_PARSE_ERROR;
+
+    QString tempname = file.section( '/', -1 );
+    if ( file.endsWith( ".kigz", false ) )
+    {
+      tempname.remove( QRegExp( "\\.[Kk][Ii][Gg][Zz]$" ) );
+    }
+    else
+      KIG_FILTER_PARSE_ERROR;
+    // reading compressed file
+    KTar* ark = new KTar( file, "application/x-gzip" );
+    ark->open( IO_ReadOnly );
+    const KArchiveDirectory* dir = ark->directory();
+//    assert( dir );
+    QStringList entries = dir->entries();
+    QStringList kigfiles = entries.grep( QRegExp( "\\.kig$" ) );
+    if ( kigfiles.count() != 1 )
+      // I throw a generic parse error here, but I should warn the user that
+      // this kig archive file doesn't contain one kig file (it contains no
+      // kig files or more than one).
+      KIG_FILTER_PARSE_ERROR;
+    const KArchiveEntry* kigz = dir->entry( kigfiles[0] );
+    if ( !kigz->isFile() )
+      KIG_FILTER_PARSE_ERROR;
+    dynamic_cast<const KArchiveFile*>( kigz )->copyTo( tempdir );
+    kdDebug() << "extracted file: " << tempdir + kigz->name() << endl
+              << "exists: " << QFile::exists( tempdir + kigz->name() ) << endl;
+
+    kigdoc.setName( tempdir + kigz->name() );
+  }
+#endif
+
+  if ( !kigdoc.open( IO_ReadOnly ) )
     KIG_FILTER_PARSE_ERROR;
-  ffile.close();
+
+  QDomDocument doc( "KigDocument" );
+  if ( !doc.setContent( &kigdoc ) )
+    KIG_FILTER_PARSE_ERROR;
+  kigdoc.close();
+
+#ifndef KIG_NO_COMPRESSED_FILES
+  // removing temp file
+  if ( iscompressed )
+    kigdoc.remove();
+#endif
+
   QDomElement main = doc.documentElement();
 
   QString version = main.attribute( "CompatibilityVersion" );
@@ -435,7 +495,18 @@ KigDocument* KigFilterNative::load07( const QString& file, const QDomElement& do
             return false;
           }
 
-          o = new ObjectTypeCalcer( type, parents );
+	  // mp: (I take the responsibility for this!) explanation: the usual ObjectTypeCalcer
+	  // constructor also "sortArgs" the parents.  I believe that this *must not* be done
+	  // when loading from a saved kig file for the following reasons:
+	  // 1. the arguments should already be in their intended order, since the file was
+	  // saved from a working hierarchy; furthermore we actually want to restore the original
+	  // hierarchy, not really to also fix possible problems with the original hierarchy;
+	  // 2. calling sortArgs could have undesirable side effects in particular situations,
+	  // since kig actually allow an ObjectType to produce different type of ObjectImp's
+	  // it may happen that the parents of an object do not satisfy the requirements
+	  // enforced by sortArgs (while moving around the free objects) but still be
+	  // perfectly valid
+          o = new ObjectTypeCalcer( type, parents, false );
         }
         else KIG_FILTER_PARSE_ERROR;
 
@@ -482,7 +553,7 @@ KigDocument* KigFilterNative::load07( const QString& file, const QDomElement& do
         {
           int ncid = tmp.toInt( &ok );
           if ( !ok ) KIG_FILTER_PARSE_ERROR;
-          if ( ncid <= 0 || id > calcers.size() )
+          if ( ncid <= 0 || ncid > calcers.size() )
             KIG_FILTER_PARSE_ERROR;
           if ( ! dynamic_cast<ObjectConstCalcer*>( calcers[ncid-1].get() ) )
             KIG_FILTER_PARSE_ERROR;
@@ -514,7 +585,7 @@ bool KigFilterNative::save07( const KigDocument& kdoc, QTextStream& stream )
   docelem.appendChild( cselem );
 
   std::vector<ObjectHolder*> holders = kdoc.objects();
-  std::vector<ObjectCalcer*> calcers = getAllParents( getCalcers( holders ) );
+  std::vector<ObjectCalcer*> calcers = getAllParents( getAllCalcers( holders ) );
   calcers = calcPath( calcers );
 
   QDomElement hierelem = doc.createElement( "Hierarchy" );
@@ -611,17 +682,72 @@ bool KigFilterNative::save( const KigDocument& data, const QString& file )
 
 bool KigFilterNative::save07( const KigDocument& data, const QString& outfile )
 {
-  QFile file( outfile );
-  if ( ! file.open( IO_WriteOnly ) )
+  // we have an empty outfile, so we have to print all to stdout
+  if ( outfile.isEmpty() )
   {
-    fileNotFound( outfile );
-    return false;
+    QTextStream stdoutstream( stdout, IO_WriteOnly );
+    return save07( data, stdoutstream );
   }
-  QTextStream stream( &file );
-  return save07( data, stream );
+#ifndef KIG_NO_COMPRESSED_FILES
+  if ( !outfile.endsWith( ".kig", false ) )
+  {
+    // the user wants to save a compressed file, so we have to save our kig
+    // file to a temp file and then compress it...
+
+    QString tempdir = KGlobal::dirs()->saveLocation( "tmp" );
+    if ( tempdir.isEmpty() )
+      return false;
+
+    QString tempname = outfile.section( '/', -1 );
+    if ( outfile.endsWith( ".kigz", false ) )
+      tempname.remove( QRegExp( "\\.[Kk][Ii][Gg][Zz]$" ) );
+    else
+      return false;
+
+    QString tmpfile = tempdir + tempname + ".kig";
+    QFile ftmpfile( tmpfile );
+    if ( !ftmpfile.open( IO_WriteOnly ) )
+      return false;
+    QTextStream stream( &ftmpfile );
+    if ( !save07( data, stream ) )
+      return false;
+    ftmpfile.close();
+
+    kdDebug() << "tmp saved file: " << tmpfile << endl;
+
+    // creating the archive and adding our file
+    KTar* ark = new KTar( outfile,  "application/x-gzip" );
+    ark->open( IO_WriteOnly );
+    ark->addLocalFile( tmpfile, tempname + ".kig" );
+    ark->close();
+
+    // finally, removing temp file
+    QFile::remove( tmpfile );
+
+    return true;
+  }
+  else
+  {
+#endif
+    QFile file( outfile );
+    if ( ! file.open( IO_WriteOnly ) )
+    {
+      fileNotFound( outfile );
+      return false;
+    }
+    QTextStream stream( &file );
+    return save07( data, stream );
+#ifndef KIG_NO_COMPRESSED_FILES
+  }
+
+  // we should never reach this point...
+  return false;
+#endif
 }
 
+/*
 bool KigFilterNative::save( const KigDocument& data, QTextStream& stream )
 {
   return save07( data, stream );
 }
+*/
