@@ -1,4 +1,3 @@
-// newscriptwizard.h
 // Copyright (C)  2003  Dominique Devriese <devriese@kde.org>
 
 // This program is free software; you can redistribute it and/or
@@ -21,6 +20,15 @@
 
 #include "newscriptwizardbase.h"
 
+#include "script-common.h"
+
+#include <ktextedit.h>
+#include <ktexteditor/document.h>
+#include <ktexteditor/highlightinginterface.h>
+#include <ktexteditor/view.h>
+
+#include <algorithm>
+
 class ScriptMode;
 
 class NewScriptWizard
@@ -36,9 +44,29 @@ public:
   void next();
   void reject();
 
+  void setText( const QString& text );
+  QString text();
+
+  void setType( ScriptType::Type type );
+
 public slots:
   void slotHelpClicked();
   void accept();
+
+  void slotUndo();
+  void slotRedo();
+  void slotCut();
+  void slotCopy();
+  void slotPaste();
+
+protected:
+  KTextEdit* textedit;
+  KTextEditor::Document* document;
+  KTextEditor::HighlightingInterface* hli;
+  KTextEditor::View* editor;
+
+  uint noHlStyle;
+  bool prevDynWordWrap;
 };
 
 #endif
