@@ -34,9 +34,9 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#include <zlib.h>
 
 #include "fli/libfli.h"
-#include "lzo/minilzo.h"
 #include "fitsrw.h"
 #include "indidevapi.h"
 #include "eventloop.h"
@@ -51,9 +51,9 @@ void getBasicData(void);
 void initDataChannel(void);
 void waitForData(int rp, int wp);
 void updateDataChannel(void *p);
-void uploadFile(char * filename);
+void uploadFile(const char* filename);
 int  findPort(void);
-int  writeFITS(char *filename, char errmsg[]);
+int  writeFITS(const char* filename, char errmsg[]);
 int  findcam(flidomain_t domain);
 int  setImageArea(char errmsg[]);
 int  manageDefaults(char errmsg[]);
@@ -96,11 +96,6 @@ extern int errno;
 
 #define getBigEndian(p) ( ((p & 0xff) << 8) | (p  >> 8))
 
-#define HEAP_ALLOC(var,size) \
-	lzo_align_t __LZO_MMODEL var [ ((size) + (sizeof(lzo_align_t) - 1)) / sizeof(lzo_align_t) ]
-
-static HEAP_ALLOC(wrkmem,LZO1X_1_MEM_COMPRESS);
-
 enum FLIFrames { LIGHT_FRAME = 0, BIAS_FRAME, DARK_FRAME, FLAT_FRAME };
 
 
@@ -221,9 +216,6 @@ void ISInit()
  INDIClients = NULL;
  nclients    = 0;
  
- if (lzo_init() != LZO_E_OK)
-   IDLog("lzo_init() failed !!!\n");
- 
  isInit = 1;
  
 }
@@ -291,29 +283,10 @@ void ISNewSwitch (const char *dev, const char *name, ISState *states, char *name
 	if (!strcmp (name, PortSP.name))
 	{
    	  PortSP.s = IPS_IDLE; 
+	  IUResetSwitches(&PortSP);
 	  IUUpdateSwitches(&PortSP, states, names, n);
 	  portSwitchIndex = getOnSwitch(&PortSP);
 	  
-	  if (portSwitchIndex == -1)
-	  {
-	    IDSetSwitch(&PortSP, "Unknown error. All switches are off.");
-	    IDLog("Unknown error. All switches are off.");
-	    return;
-	  }
-	  
-	  for (i=0 ; i < n ; i++)
-	  {
-	    sp = IUFindSwitch(&PortSP, names[i]);
-	    
-	    if (!sp)
-	    {
-	      IDSetSwitch(&PortSP, "Unknown error. %s is not a member of %s property.", names[0], name);
-	      return;
-	    }
-	    
-	    sp->s = states[i];
-	  }
-	  
 	  PortSP.s = IPS_OK; 
 	  IDSetSwitch(&PortSP, NULL);
 	  return;
@@ -322,6 +295,7 @@ void ISNewSwitch (const char *dev, const char *name, ISState *states, char *name
 	/* Connection */
 	if (!strcmp (name, PowerSP.name))
 	{
+	  IUResetSwitches(&PowerSP);
 	  IUUpdateSwitches(&PowerSP, states, names, n);
    	  connectCCD();
 	  return;
@@ -419,7 +393,7 @@ void ISNewNumber (const char *dev, const char *name, double values[], char *name
       long err;
       int i;
       INumber *np;
-      char errmsg[1024];
+      char errmsg[ERRMSG_SIZE];
 
 	/* ignore if not ours */
 	if (dev && strcmp (dev, mydev))
@@ -556,14 +530,14 @@ void ISNewNumber (const char *dev, const char *name, double values[], char *name
       
       if (nset < 4)
       {
-        IDSetNumber(&FrameNP, "Invalid range. Valid range is (0,0) - (%0.f,%0.f)", FLICam->width, FLICam->height);
-	IDLog("Invalid range. Valid range is (0,0) - (%0.f,%0.f)", FLICam->width, FLICam->height);
+        IDSetNumber(&FrameNP, "Invalid range. Valid range is (0,0) - (%0d,%0d)", FLICam->width, FLICam->height);
+	IDLog("Invalid range. Valid range is (0,0) - (%0d,%0d)", FLICam->width, FLICam->height);
 	return; 
       }
       
       if (setImageArea(errmsg))
       {
-        IDSetNumber(&FrameNP, errmsg);
+        IDSetNumber(&FrameNP, "%s", errmsg);
 	return;
       }
       	    
@@ -600,8 +574,8 @@ void ISNewNumber (const char *dev, const char *name, double values[], char *name
       {
         if (values[i] < 1 || values[i] > MAX_X_BIN)
 	{
-	  IDSetNumber(&BinningNP, "Error: Valid X bin values are from 1 to %d", MAX_X_BIN);
-	  IDLog("Error: Valid X bin values are from 1 to %d", MAX_X_BIN);
+	  IDSetNumber(&BinningNP, "Error: Valid X bin values are from 1 to %g", MAX_X_BIN);
+	  IDLog("Error: Valid X bin values are from 1 to %g", MAX_X_BIN);
 	  return;
 	}
 	
@@ -618,8 +592,8 @@ void ISNewNumber (const char *dev, const char *name, double values[], char *name
       {
         if (values[i] < 1 || values[i] > MAX_Y_BIN)
 	{
-	  IDSetNumber(&BinningNP, "Error: Valid Y bin values are from 1 to %d", MAX_Y_BIN);
-	  IDLog("Error: Valid X bin values are from 1 to %d", MAX_Y_BIN);
+	  IDSetNumber(&BinningNP, "Error: Valid Y bin values are from 1 to %g", MAX_Y_BIN);
+	  IDLog("Error: Valid X bin values are from 1 to %g", MAX_Y_BIN);
 	  return;
 	}
 	
@@ -637,7 +611,7 @@ void ISNewNumber (const char *dev, const char *name, double values[], char *name
      if (setImageArea(errmsg))
      {
        IDSetNumber(&BinningNP, errmsg, NULL);
-       IDLog(errmsg);
+       IDLog("%s", errmsg);
        return;
      }
      
@@ -817,8 +791,8 @@ int setImageArea(char errmsg[])
    
    if ( (err = FLISetImageArea(fli_dev, x_1, y_1, x_2, y_2) ))
    {
-     sprintf(errmsg, "FLISetImageArea() failed. %s.\n", strerror((int)-err));
-     IDLog(errmsg, NULL);
+     snprintf(errmsg, ERRMSG_SIZE, "FLISetImageArea() failed. %s.\n", strerror((int)-err));
+     IDLog("%s", errmsg);
      return -1;
    }
    
@@ -832,13 +806,13 @@ int grabImage()
 {
   long err;
   int img_size,i, fd;
-  char errmsg[1024];
+  char errmsg[ERRMSG_SIZE];
   char filename[] = "/tmp/fitsXXXXXX";
   
    if ((fd = mkstemp(filename)) < 0)
    { 
-    IDMessage(mydev, "Error making temporary filename.", NULL);
-    IDLog("Error making temporary filename.\n", NULL);
+    IDMessage(mydev, "Error making temporary filename.");
+    IDLog("Error making temporary filename.\n");
     return -1;
    }
    close(fd);
@@ -849,12 +823,12 @@ int grabImage()
   
   if (FLIImg->img == NULL)
   {
-    IDMessage(mydev, "Not enough memory to store image.", NULL);
-    IDLog("Not enough memory to store image.\n", NULL);
+    IDMessage(mydev, "Not enough memory to store image.");
+    IDLog("Not enough memory to store image.\n");
     return -1;
   }
   
-  for (i=0; i < FLIImg->width ; i++)
+  for (i=0; i < FLIImg->height ; i++)
   {
     if ( (err = FLIGrabRow(fli_dev, &FLIImg->img[i * FLIImg->width], FLIImg->width)))
     {
@@ -880,7 +854,7 @@ int grabImage()
  
 }
 
-int writeFITS(char *filename, char errmsg[])
+int writeFITS(const char* filename, char errmsg[])
 {
   FITS_FILE* ofp;
   int i, j, bpp, bpsl, width, height;
@@ -890,7 +864,7 @@ int writeFITS(char *filename, char errmsg[])
   ofp = fits_open (filename, "w");
   if (!ofp)
   {
-    sprintf(errmsg, "Error: cannot open file for writing.");
+    snprintf(errmsg, ERRMSG_SIZE, "Error: cannot open file for writing.");
     return (-1);
   }
   
@@ -903,12 +877,12 @@ int writeFITS(char *filename, char errmsg[])
   hdu = create_fits_header (ofp, width, height, bpp);
   if (hdu == NULL)
   {
-     sprintf(errmsg, "Error: creating FITS header failed.");
+     snprintf(errmsg, ERRMSG_SIZE, "Error: creating FITS header failed.");
      return (-1);
   }
   if (fits_write_header (ofp, hdu) < 0)
   {
-    sprintf(errmsg, "Error: writing to FITS header failed.");
+    snprintf(errmsg, ERRMSG_SIZE, "Error: writing to FITS header failed.");
     return (-1);
   }
   
@@ -932,7 +906,7 @@ int writeFITS(char *filename, char errmsg[])
   
   if (ferror (ofp->fp))
   {
-    sprintf(errmsg, "Error: write error occured");
+    snprintf(errmsg, ERRMSG_SIZE, "Error: write error occured");
     return (-1);
   }
  
@@ -942,7 +916,7 @@ int writeFITS(char *filename, char errmsg[])
  ExposeTimeNP.s = IPS_OK;
  /*IDSetNumber(&ExposeTimeNP, "FITS image written to %s", filename);
  IDLog("FITS image written to '%s'\n", filename);*/
- IDSetNumber(&ExposeTimeNP, "Loading FITS image...");
+ IDSetNumber(&ExposeTimeNP, NULL);
  IDLog("Loading FITS image...\n");
  
  uploadFile(filename);
@@ -951,18 +925,20 @@ int writeFITS(char *filename, char errmsg[])
 
 }
 
-void uploadFile(char * filename)
+void uploadFile(const char* filename)
 {
    FILE * fitsFile;
    char frameSize[FRAME_ILEN];
    unsigned char *fitsData, *compressedData;
    int n=0,r=0;
-   unsigned int i =0, j = 0, totalBytes, compressedBytes = 0, nr = 0, frameLen;
+   unsigned int i =0, j = 0, nr = 0, frameLen;
+   uLongf compressedBytes=0;
+   uLong  totalBytes;
    struct stat stat_p; 
  
    if ( -1 ==  stat (filename, &stat_p))
    { 
-     IDLog(" Error occoured attempting to stat %s\n", filename); 
+     IDLog(" Error occoured attempting to stat file.\n"); 
      return; 
    }
    
@@ -993,18 +969,17 @@ void uploadFile(char * filename)
      }
    }
    
-   /* #2 Compress it */
-   r = lzo1x_1_compress(fitsData, totalBytes, compressedData, &compressedBytes, wrkmem);
-   if (r != LZO_E_OK)
-      /*IDLog("compressed %lu bytes into %lu bytes\n", (long) totalBytes, (long) compressedBytes);
-   else*/
+   compressedBytes = sizeof(char) * totalBytes + totalBytes / 64 + 16 + 3;
+     
+   r = compress2(compressedData, &compressedBytes, fitsData, totalBytes, 9);
+   if (r != Z_OK)
    {
  	/* this should NEVER happen */
  	IDLog("internal error - compression failed: %d\n", r);
 	return;
    }
    
-   snprintf(frameSize, FRAME_ILEN, "<Data type='FITS' size='%d' compsize='%d' />", totalBytes, compressedBytes);
+   snprintf(frameSize, FRAME_ILEN, "<Data type='FITS' size='%ld' compsize='%ld' />", totalBytes, compressedBytes);
    frameLen = strlen(frameSize);
    r = 0;
    
@@ -1071,7 +1046,7 @@ void handleExposure(void *p)
  
    ExposeTimeNP.s = IPS_BUSY;
 		  
-   IDSetNumber(&ExposeTimeNP, "Taking a %g seconds frame...", ExposeTimeN[0].value);
+   IDSetNumber(&ExposeTimeNP, "Taking a %g seconds frame...", FLIImg->expose / 1000.);
    
    IDLog("Taking a frame...\n");
 }
@@ -1185,7 +1160,7 @@ int manageDefaults(char errmsg[])
   IDLog("Setting default exposure time of %d ms.\n", exposeTimeMS);
   if ( (err = FLISetExposureTime(fli_dev, exposeTimeMS) ))
   {
-    sprintf(errmsg, "FLISetExposureTime() failed. %s.\n", strerror((int)-err));
+    snprintf(errmsg, ERRMSG_SIZE, "FLISetExposureTime() failed. %s.\n", strerror((int)-err));
     IDLog(errmsg, NULL);
     return -1;
   }
@@ -1193,7 +1168,7 @@ int manageDefaults(char errmsg[])
   /* Default frame type is NORMAL */
   if ( (err = FLISetFrameType(fli_dev, FLI_FRAME_TYPE_NORMAL) ))
   {
-    sprintf(errmsg, "FLISetFrameType() failed. %s.\n", strerror((int)-err));
+    snprintf(errmsg, ERRMSG_SIZE, "FLISetFrameType() failed. %s.\n", strerror((int)-err));
     IDLog(errmsg, NULL);
     return -1;
   }
@@ -1201,7 +1176,7 @@ int manageDefaults(char errmsg[])
   /* X horizontal binning */
   if ( (err = FLISetHBin(fli_dev, BinningN[0].value) ))
   {
-    sprintf(errmsg, "FLISetBin() failed. %s.\n", strerror((int)-err));
+    snprintf(errmsg, ERRMSG_SIZE, "FLISetBin() failed. %s.\n", strerror((int)-err));
     IDLog(errmsg, NULL);
     return -1;
   }
@@ -1209,7 +1184,7 @@ int manageDefaults(char errmsg[])
   /* Y vertical binning */
   if ( (err = FLISetVBin(fli_dev, BinningN[1].value) ))
   {
-    sprintf(errmsg, "FLISetVBin() failed. %s.\n", strerror((int)-err));
+    snprintf(errmsg, ERRMSG_SIZE, "FLISetVBin() failed. %s.\n", strerror((int)-err));
     IDLog(errmsg, NULL);
     return -1;
   }
@@ -1231,8 +1206,11 @@ int getOnSwitch(ISwitchVectorProperty *sp)
 {
   int i=0;
  for (i=0; i < sp->nsp ; i++)
+ {
+   /*IDLog("Switch %s is %s\n", sp->sp[i].name, sp->sp[i].s == ISS_ON ? "On" : "Off");*/
      if (sp->sp[i].s == ISS_ON)
       return i;
+ }
 
  return -1;
 }
@@ -1293,7 +1271,7 @@ int checkPowerT(ITextVectorProperty *tp)
 void connectCCD()
 {
   long err;
-  char errmsg[1024];
+  char errmsg[ERRMSG_SIZE];
  
   IDLog ("In ConnectCCD\n");
    
@@ -1301,6 +1279,7 @@ void connectCCD()
   switch (PowerS[0].s)
   {
     case ISS_ON:
+      IDLog("Current portSwitch is %d\n", portSwitchIndex);
       IDLog("Attempting to find the camera in domain %ld\n", Domains[portSwitchIndex]);
       if (findcam(Domains[portSwitchIndex]))
       {
@@ -1332,7 +1311,7 @@ void connectCCD()
       if (manageDefaults(errmsg))
       {
         IDMessage(mydev, errmsg, NULL);
-	IDLog(errmsg);
+	IDLog("%s", errmsg);
 	return;
       }
       
@@ -1495,7 +1474,7 @@ FITS_HDU_LIST * create_fits_header (FITS_FILE *ofp, uint width, uint height, uin
  fits_add_card (hdulist, temp_s);
  fits_add_card (hdulist, expose_s);
  fits_add_card (hdulist, pixel_s);
- fits_add_card (hdulist, "INSTRUME= 'Finger Lakes Instruments'");
+ fits_add_card (hdulist, ( char* ) "INSTRUME= 'Finger Lakes Instruments'");
  fits_add_card (hdulist, obsDate);
   
  return (hdulist);
@@ -1632,7 +1611,7 @@ int findPort()
   /* bind to given port for any IP address */
   memset (&serv_socket, 0, sizeof(serv_socket));
   serv_socket.sin_family = AF_INET;
-  serv_socket.sin_addr.s_addr = htonl (INADDR_ANY);
+  serv_socket.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
   
   for (i=0; i < 100; i++)
   {
@@ -1675,7 +1654,7 @@ void waitForData(int rp, int wp)
 	/* bind to given port for any IP address */
 	memset (&serv_socket, 0, sizeof(serv_socket));
 	serv_socket.sin_family = AF_INET;
-	serv_socket.sin_addr.s_addr = htonl (INADDR_ANY);
+	serv_socket.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
 	
 	serv_socket.sin_port = htons ((unsigned short)DataPort);
 	if (setsockopt(sfd,SOL_SOCKET,SO_REUSEADDR,&reuse,sizeof(reuse)) < 0){
