@@ -189,7 +189,7 @@ extern int nXMLAtt (XMLEle *ep);
     \param name the name of the XML attribute to retrieve its value.
     \return the value string of an XML element on success. NULL on failure.
 */
-extern char *findXMLAttValu (XMLEle *ep, char *name);
+extern const char *findXMLAttValu (XMLEle *ep, char *name);
 
 /** \brief Handy wrapper to read one xml file.
     \param fp pointer to FILE to read.
@@ -242,7 +242,7 @@ extern void prXMLEle (FILE *fp, XMLEle *e, int level);
  */
 
 /* For RCS Only -- Do Not Edit
- * @(#) $RCSfile: lilxml.h,v $ $Date: 2004/06/07 05:59:20 $ $Revision: 1.6 $ $Name:  $
+ * @(#) $RCSfile: lilxml.h,v $ $Date: 2005/02/15 18:02:39 $ $Revision: 1.7 $ $Name:  $
  */
 
 #endif	/* LILXML_H */
